/**************************************************************************************
*									KoSi_Gen.h
*
*
* GGC-Version:			4.5.1
*
* AVR-Studio Version:	5.0.1163
*
* Autor: 				Rick
*
* Getestet:						
*
* Letzte nderung:		
*
*
*
* 
***************************************************************************************/

//********************* Defines *******************************************************
#ifndef KOSI_GEN_H_
#define KOSI_GEN_H_
#endif

#ifndef UART_BAUD_RATE
#define UART_BAUD_RATE	9600					// Baudrate fr UART
#endif

#ifndef XTAL
#define XTAL 16000000L							// Quarzfrequenz fr Berechnung in Herz
#endif

#ifndef F_CPU
#define F_CPU 16000000L							// Quarzfrequenz fr delay library
#endif





#include <avr/io.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>

#include "timer1.h"
#include "lcd.h"






//******************** globale Variablen *********************************************

int8_t  second_countdown = 0;			// Variablen fr Countdown
int8_t  hour_countdown = 0;
int8_t  minute_countdown = 10;		
uint8_t gen_on = 0;						// Variable fr Gert EIN=1 / AUS=0
uint8_t ac_period = 1;					// Variable fr Pulslnge Rechteckwechselspg [sec]
uint8_t ac_time = 0;					// Zhlvariable fr Pulslnge
uint8_t current_mode = 0;				// Variable fr Spannungsart:
										// current_mode = 1 -> AC
										// current_mode = 0 -> DC

uint32_t voltage_source;				// Variable fr Quellenspannung
uint32_t voltage_electrodes = 64000;			// Elektroden-Spannung
uint32_t current_electrodes = 20000000;			// Elektroden-Strom

uint8_t button1, button2;
uint16_t adc_value_button;

/******************************* Konstanten *******************************************
**************************************************************************************/


#define VAREF	2130L		// Referenzspannung fr AD-Wandler in mV (2,5V -> 2500)

#define K		25L		// Teilerfaktor fr Spannungsteiler zu Spannungsmessung
						// Bsp.: Verh.: 1:25 -> Wert=25
						
#define R_SHUNT 120L	// Widerstandswert fr Shunt (Strommessung-Elektroden) im Ohm


#define K_VOLTAGE		(VAREF * K / 1024)
// Korrekturfaktor fr Spg.-Messung, Ergebnis in mV !

#define K_CURRENT		( (VAREF * 1000000) / (R_SHUNT * 1024) )
// Korrekturfaktor fr Strom-Messung, Ergebnis in uA !




//********************** Messkanle am AD-Wandler ************************************

#define CHANNEL_VOLTAGE_SOURCE 0	 // AD-Wandler-Kanal fr Messung Quellenspg.

#define CHANNEL_VOLTAGE_ELECTRODES 1 // AD-Wandler-Kanal fr Messung Elektrodenspg.

#define CHANNEL_CURRENT_ELECTRODES 2 // AD-Wandler-Kanal fr Messung Elektrodenstrom


//*********************** Ausgnge ***************************************************

#define PORT_MAIN_SWITCH	PORTC				// Hauptschalter fr Generator
#define PIN_MAIN_SWITCH		PD0

#define PORT_AC_SWITCH		PORTC				// Ausgang fr Polarittswechsel
#define PIN_AC_SWITCH		PD1

#define PORT_GREEN_LED		PORTC				// Ausgang fr grne LED
#define PIN_GREEN_LED		PC2

#define PORT_RED_LED		PORTC				// Ausgang fr rote LED
#define PIN_RED_LED			PC3


 																	
																			
//---------------------- Funktionsprototypen -------------------------------------------
void adc_init(void);
uint16_t adc_read_avg (uint8_t channel , uint8_t average);
uint32_t read_voltage_source(void);
uint32_t read_voltage_electrodes(void);
uint32_t read_current_electrodes(void);
uint8_t read_button(void);
void round_32bit (char* string , uint8_t index_digit);
void print_lcd (char* string, uint8_t index_start, uint8_t index_digit_1, uint8_t frac);
void print_uart (char* string ,uint8_t index_start, uint8_t index_digit_1, uint8_t frac);
void display_lcd_page1(void);





/***************************************************************************************
*
*Funktionsname:		adc_init
*
*Rckgabe:			keine
*
*Parameter:			keine
*
*Aufgabe:			Initialisierung des AD-Converters
*
*getestet: 
*
***************************************************************************************/

void adc_init(void)
{
	
	ADMUX &= ~( (1<<REFS1) | (1<<REFS0) ); 	
	//Externe Referenzspannung nutzen -> REFS1 uns REFS0 auf 0 setzen
										
	ADCSRA |= (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
	/* Maximale Wandlerfrequenz = 200kHz !
	Dadurch ergibt sich ein Teilungsfaktor von F_CPU/200kHz = 80 bei 16Mhz Quarz
	Nchster Faktor ist 128
	Somit ergibt sich eine Wandlerfrequenz von 125kHz
	ADPS2, ADPS1, ADPS0 auf 1 setzen*/
	
	ADCSRA &= ~(1<<ADIE);
	// kein Interrupt, wenn Wandlung beendet -> ADIE=0
	
	ADCSRA |= (1<<ADEN);	//ADC durch ADEN=1 aktivieren
		
}
	

/***************************************************************************************
*
*Funktionsname:		adc_Read_Avg
*
*Rckgabe:			Mittel der ADC-Messungen
*
*Parameter:			Nummer des ADC-Kanals, Anzahl der Wandlungen
*
*Aufgabe:			Stellt den Kanal des ADC ein, fhrt eine gewnschte 
*					Anzahl von Wandlungen durch und errechnet dessen Mittelwert	
*
*Hinweis:			Anzahl der Messungen darf 34 nicht bersteigen, ansonsten mu  
*					"result" eine 32bit Variable sein! (2^16 / 2^10 = 34,7)
*
* getestet: 
*
***************************************************************************************/
	
uint16_t adc_read_avg (uint8_t channel , uint8_t average)
{

	uint16_t result;
	

	ADMUX = ( (ADMUX & ~(0x1F)) | (channel & 0x1F) );
	/* Kanal whlen, ohne andere Bits zu verndern

	 0bxxx00000		ADMUX & ~(0x1F) -> Channel Bits auf 0 setzen
	 0bxxx00001		1 & 0x1F  (=channel=1)
	____________	oder - Verknpfung
	 0bxxx00001									*/

	 /* Nach der Kanalwahl des ADC wird ein "Dummy Readout" zur Sicherheit durchgefhrt.
		Man startet eine Wandlung, liest den Wert aus und verwirft diesen */
	
	ADCSRA |= (1<<ADSC); // ADSC=1 Wandlung starten
	
	while (ADCSRA & (1<<ADSC)) {}
	//Solange ADSC = 1 ist, ist die Wandlung noch nicht abgeschlossen
		

	result = ADCW;					
	// Ergebnis der Wandlung auslesen, sonst wird Ergebnis der nchsten Wandlung nicht 
	// bernommen

	result = 0;								// Ergebnis verwerfen

	// Durchfhrung mehrerer Messungen und Ergebnise aufaddieren:
	for (uint8_t i=0; i < average; i++)
		{
			ADCSRA |= (1<<ADSC);			// 1ne Wandlung durchfhren
			while (ADCSRA & (1<<ADSC)) {}	// Warten bis Wandlung beendet
			result += ADCW;					// Ergebnisse aufaddieren
		}

	result = result / average;				// Mittelwert bilden
	return result;							// Ergebnis zurck geben
	
}





/***************************************************************************************
*
*Funktionsname:		read_voltage_source
*
*Rckgabe:			Spannung der Quelle in mV
*
*Parameter:			keine
*
*Aufgabe:			Liest die Quellen-Spannung aus und gibt diese zurck.
*
*getestet: 
*
***************************************************************************************/

uint32_t read_voltage_source(void)
{
	
	uint16_t adc_value;
	uint32_t value_voltage_source;
	
	adc_value = adc_read_avg( CHANNEL_VOLTAGE_SOURCE , 4);	
	//Auslesen des AD-Wertes fr die Quellenspg; 4 Messungen
	
	//value_voltage_source = (uint16_t) ( (adc_value * VAREF * K) / 1024 );
	
	value_voltage_source = (uint32_t) ( adc_value * K_VOLTAGE );
	// Quellenspannung = (ADC-Wert * Korrekturfaktor fr Spg.-Messung
	
	return value_voltage_source;	// Quellenspg. in mV zurckgeben
	
}






/***************************************************************************************
*
*Funktionsname:		read_voltage_electrodes
*
*Rckgabe:			Spannung der Elektroden in mV
*
*Parameter:			keine
*
*Aufgabe:			Liest die Elektroden-Spannung aus und gibt diese zurck.
*
*getestet: 
*
***************************************************************************************/

uint32_t read_voltage_electrodes(void)
{
	
	uint16_t adc_value, adc_value_1 = 1024, adc_value_2 = 0;
	uint32_t value_voltage_electrodes;
	
	//adc_value_1 = adc_read_avg( CHANNEL_VOLTAGE_ELECTRODES , 4);	
	/*Auslesen des AD-Wertes fr die Elektrodenspg inkl. Spannungsabfall am Shunt!,
																		4 Messungen */
	
	//adc_value_2 = adc_read_avg( CHANNEL_CURRENT_ELECTRODES , 4);
	// Auslesen des AD-Wertes fr den Spaannungsabfall am Shunt, 4 Messungen
	
	adc_value = adc_value_1 - adc_value_2;
	// Elektrodenspg. = Spg. gesamt - Spg. am Shunt
	
	//value_voltage_electrodes = (uint16_t) ( (adc_value * VAREF * K) / 1024 );
	
	value_voltage_electrodes = ( adc_value * K_VOLTAGE);
/* Elektrodenspg. = (ADC-Wert * Referenzspg. AD-Wandler * Faktor-Spannungsteiler) /
																			  2^10bit */
	
	return value_voltage_electrodes;	// Elektrodenspg. in mV zurckgeben
	
}






/***************************************************************************************
*
*Funktionsname:		read_current_electrodes
*
*Rckgabe:			Strom durch die Elektroden in !! uA !!
*
*Parameter:			keine
*
*Aufgabe:			Liest den Elektroden-Strom aus und gibt diesen zurck.
*
*getestet: 
*
***************************************************************************************/

uint32_t read_current_electrodes(void)
{
	
	uint16_t adc_value= 1024;
	uint32_t value_current_electrodes;
	
	//adc_value = adc_read_avg( CHANNEL_CURRENT_ELECTRODES , 4);	
	//Auslesen des AD-Wertes fr den Spg.-abfall am Shunt, 4 Messungen
	
	//value_voltage_shunt = (uint16_t) ( (adc_value * VAREF) / 1024 );
	// Spg.-abfall Shunt = (ADC-Wert * Referenzspg. AD-Wandler) / 2^10bit
	
	//value_current_electrodes = value_voltage_shunt / R_SHUNT;
	// Elektrodenstrom = Spg.-abfall Shunt / Widerstand Shunt [Ohm]
	
	
	value_current_electrodes = (adc_value * K_CURRENT);
	// Elektrodenstrom [uA]! = ADC-Wert * Korrekturfaktor fr Strommessung
	
	
	return value_current_electrodes;	// Elektrodenstrom in uA! zurckgeben
	
}







/***************************************************************************************
*
*Funktionsname:		read_button
*
*Rckgabe:			Tastennummer (1 bis 5), bei Fehler 0
*
*Parameter:			keine
*
*Aufgabe:			Ermittelt die gedrckte Taste
*
*
* getestet: 
*
***************************************************************************************/

uint8_t read_button(void)
{
	//uint16_t adc_value_button;
	uint8_t button = 0;
	
	adc_value_button = adc_read_avg(7,4);
	//ADC-Wert fr Tasten auslesen, Channel 7 / 4 Messungen
	
	// Wertebereich fr AREF = 2,13V !!
	
	if( (adc_value_button >= 0) && (adc_value_button <= 10 ) ) button = 0; // keine Taste gedrckt
	
	else if( (adc_value_button >= 750) && (adc_value_button <= 770) ) button = 1;
	//liegt der ADC-Wert innerhalb des Fensters fr Taster 1 ?
	
	else if ( (adc_value_button >= 600) && (adc_value_button <= 620) ) button = 2;		// Taste 2
	else if ( (adc_value_button >= 450) && (adc_value_button <= 470) ) button = 3;		// Taste 3
	else if ( (adc_value_button >= 297) && (adc_value_button <= 317) ) button = 4;		// Taste 4	
	else if ( (adc_value_button >= 144) && (adc_value_button <= 164) ) button = 5;		// Taste 5
	else button = 99; // Fehler bei Tastenerkennung !!
		
		
	return button;		// Tastennummer zurckgeben
		
}






/***************************************************************************************
*
*Funktionsname:		display_lcd_page1
*
*Rckgabe:			keine
*
*Parameter:			keine
*
*Aufgabe:			Gibt die 1. Anzeige-Seite am Display aus
*
*
* getestet: 
***************************************************************************************/

void display_lcd_page1(void)
{
	char string[12];						// Platz fr 11 Zeichen + /0
	
	
	//lcd_clrscr();							// Display lschen und cursor home
	lcd_gotoxy(0,0);
	
	itoa(voltage_electrodes, string, 10);	// Elektrodenspg. in ASCII umwandeln
	//round_32bit(string+1, 9);				// Wert runden
	//print_lcd(string, 6, 8, 1);				// Spg. formatiert am LCD ausgeben
	lcd_putc('V');							// Volt Symbol ausgeben
	lcd_putc(' ');							// Leerstelle
	
	itoa(current_electrodes, string, 10);	// Elektrodenstrom in ASCII umwandeln
	//round_32bit(string+1, 7);				// Wert runden
	//print_lcd(string, 3, 5, 2);				// Strom formatiert am LCD ausgeben
	lcd_puts("mA");							// Strom Symbol ausgeben
	lcd_putc(' ');							// Leerstelle
	
	if (current_mode == 0)					// Stromart anzeigen
		lcd_puts("DC");	
	else
		lcd_puts("AC");	
		
	lcd_gotoxy(0, 1);						// nchste Zeile, 1 Zeichen
	
	itoa(hour_countdown, string, 10);		// Stunden in ASCII umwandeln
	if(hour_countdown < 10)					// wenn Stunden kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Stunden ausgeben
	
	lcd_putc(':');							// Doppelpunkt ausgeben
	
	itoa(minute_countdown, string, 10);		// Minuten in ASCII umwandeln
	if(minute_countdown < 10)				// wenn Minuten kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Minuten ausgeben
	
	lcd_putc(':');							// Doppelpunkt ausgeben	
	
	itoa(second_countdown, string, 10);		// Sekunden in ASCII umwandeln
	if(second_countdown < 10)				// wenn Sekunden kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Sekunden ausgeben
	
	lcd_puts("   ");						// 3 Leerstellen
	
	itoa(ac_period, string, 10);			// AC-Periodendauer in ASCII umwandeln
	if(ac_period < 10)						// wenn Sekunden kleiner 10
		lcd_putc('0');						// Null ausgeben
	lcd_puts(string);						// Sekunden ausgeben
	lcd_puts("sec");						// Symbol fr Sekunden ausgeben
	
}		
	
	
	
	
	
	
